@@ -23,6 +23,7 @@ Options:
 -d/--daemon -- run as a proper daemon; fork a subprocess, close files etc.
 -f/--forever -- run forever (by default, exit when backoff limit is exceeded)
 -h/--help -- print this usage message and exit
+-t/--transcript FILE -- log file where to redirect stdout and stderr
 -l/--logfile -- log file to be read by logtail command
 -p/--program PROGRAM -- the program to run
 -S/--schema XML Schema -- XML schema for configuration file
@@ -37,6 +38,7 @@ action [arguments] -- see below
 Actions are commands like "start", "stop" and "status".  Use the
 action "help" to find out about available actions.
 """
+from __future__ import print_function
 
 import os
 import os.path
@@ -127,9 +129,9 @@ class ZDCmd(cmd.Cmd):
                 args = eval(s, {"__builtins__": {}})
                 program = self.options.program
                 if args[:len(program)] != program:
-                    print "WARNING! zdrun is managing a different program!"
-                    print "our program   =", program
-                    print "daemon's args =", args
+                    print("WARNING! zdrun is managing a different program!")
+                    print("our program   =", program)
+                    print("daemon's args =", args)
 
         if options.configroot is not None:
             env = getattr(options.configroot, 'environment', None)
@@ -216,18 +218,19 @@ class ZDCmd(cmd.Cmd):
         sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
         try:
             sock.connect(self.options.sockname)
-            sock.send(action + "\n")
+            sock.send(action.encode() + b"\n")
             sock.shutdown(1) # We're not writing any more
-            response = ""
+            response = b""
             while 1:
                 data = sock.recv(1000)
                 if not data:
                     break
                 response += data
-            sock.close()
-            return response
-        except socket.error, msg:
+            return response.decode()
+        except socket.error as msg:
             return None
+        finally:
+            sock.close()
 
     zd_testing = 0
     def get_status(self):
@@ -266,21 +269,21 @@ class ZDCmd(cmd.Cmd):
                 if self.get_status():
                     was_running = True
                 elif (was_running or n > 10) and not cond(n):
-                    print "\ndaemon manager not running"
+                    print("\ndaemon manager not running")
                     return 1
 
         except KeyboardInterrupt:
-            print "^C"
-        print "\n" + msg % self.__dict__
+            print("^C")
+        print("\n" + msg % self.__dict__)
 
 
     def help_help(self):
-        print "help          -- Print a list of available actions."
-        print "help <action> -- Print help for <action>."
+        print("help          -- Print a list of available actions.")
+        print("help <action> -- Print help for <action>.")
 
     def _start_cond(self, n):
         if (n > self.options.start_timeout):
-            print '\nProgram took too long to start'
+            print('\nProgram took too long to start')
             sys.exit(1)
         return self.zd_pid and not self.zd_testing
 
@@ -299,6 +302,7 @@ class ZDCmd(cmd.Cmd):
             args += self._get_override("-f", "forever", flag=1)
             args += self._get_override("-s", "sockname")
             args += self._get_override("-u", "user")
+            args += self._get_override("-t", "transcript")
             if self.options.umask:
                 args += self._get_override("-m", "umask",
                                            oct(self.options.umask))
@@ -315,7 +319,7 @@ class ZDCmd(cmd.Cmd):
         elif not self.zd_pid:
             self.send_action("start")
         else:
-            print "daemon process already running; pid=%d" % self.zd_pid
+            print("daemon process already running; pid=%d" % self.zd_pid)
             return
         if self.options.daemon:
             return self.awhile(
@@ -349,30 +353,30 @@ class ZDCmd(cmd.Cmd):
         return args
 
     def help_start(self):
-        print "start -- Start the daemon process."
-        print "         If it is already running, do nothing."
+        print("start -- Start the daemon process.")
+        print("         If it is already running, do nothing.")
 
     def do_stop(self, arg):
         self.get_status()
         if not self.zd_up:
-            print "daemon manager not running"
+            print("daemon manager not running")
         elif not self.zd_pid:
-            print "daemon process not running"
+            print("daemon process not running")
         else:
             self.send_action("stop")
             self.awhile(lambda n: not self.zd_pid, "daemon process stopped")
 
     def do_reopen_transcript(self, arg):
         if not self.zd_up:
-            print "daemon manager not running"
+            print("daemon manager not running")
         elif not self.zd_pid:
-            print "daemon process not running"
+            print("daemon process not running")
         else:
             self.send_action("reopen_transcript")
 
     def help_stop(self):
-        print "stop -- Stop the daemon process."
-        print "        If it is not running, do nothing."
+        print("stop -- Stop the daemon process.")
+        print("        If it is not running, do nothing.")
 
     def do_restart(self, arg):
         self.get_status()
@@ -385,7 +389,7 @@ class ZDCmd(cmd.Cmd):
                         "daemon process restarted, pid=%(zd_pid)d")
 
     def help_restart(self):
-        print "restart -- Stop and then start the daemon process."
+        print("restart -- Stop and then start the daemon process.")
 
     def do_kill(self, arg):
         if not arg:
@@ -394,149 +398,150 @@ class ZDCmd(cmd.Cmd):
             try:
                 sig = int(arg)
             except: # int() can raise any number of exceptions
-                print "invalid signal number", `arg`
+                print("invalid signal number", repr(arg))
                 return
         self.get_status()
         if not self.zd_pid:
-            print "daemon process not running"
+            print("daemon process not running")
             return
-        print "kill(%d, %d)" % (self.zd_pid, sig)
+        print("kill(%d, %d)" % (self.zd_pid, sig))
         try:
             os.kill(self.zd_pid, sig)
-        except os.error, msg:
-            print "Error:", msg
+        except os.error as msg:
+            print("Error:", msg)
         else:
-            print "signal %d sent to process %d" % (sig, self.zd_pid)
+            print("signal %d sent to process %d" % (sig, self.zd_pid))
 
     def help_kill(self):
-        print "kill [sig] -- Send signal sig to the daemon process."
-        print "              The default signal is SIGTERM."
+        print("kill [sig] -- Send signal sig to the daemon process.")
+        print("              The default signal is SIGTERM.")
 
     def do_wait(self, arg):
         self.awhile(lambda n: not self.zd_pid, "daemon process stopped")
         self.do_status()
 
     def help_wait(self):
-        print "wait -- Wait for the daemon process to exit."
+        print("wait -- Wait for the daemon process to exit.")
 
     def do_status(self, arg=""):
         status = 0
         if arg not in ["", "-l"]:
-            print "status argument must be absent or -l"
+            print("status argument must be absent or -l")
             return 1
         self.get_status()
         if not self.zd_up:
-            print "daemon manager not running"
+            print("daemon manager not running")
             status = 3
         elif not self.zd_pid:
-            print "daemon manager running; daemon process not running"
+            print("daemon manager running; daemon process not running")
         else:
-            print "program running; pid=%d" % self.zd_pid
+            print("program running; pid=%d" % self.zd_pid)
         if arg == "-l" and self.zd_status:
-            print self.zd_status
+            print(self.zd_status)
         return status
 
     def help_status(self):
-        print "status [-l] -- Print status for the daemon process."
-        print "               With -l, show raw status output as well."
+        print("status [-l] -- Print status for the daemon process.")
+        print("               With -l, show raw status output as well.")
 
     def do_show(self, arg):
         if not arg:
             arg = "options"
         try:
             method = getattr(self, "show_" + arg)
-        except AttributeError, err:
-            print err
+        except AttributeError as err:
+            print(err)
             self.help_show()
             return
         method()
 
     def show_options(self):
-        print "zdctl/zdrun options:"
-        print "schemafile:  ", repr(self.options.schemafile)
-        print "configfile:  ", repr(self.options.configfile)
-        print "zdrun:       ", repr(self.options.zdrun)
-        print "python:      ", repr(self.options.python)
-        print "program:     ", repr(self.options.program)
-        print "backofflimit:", repr(self.options.backofflimit)
-        print "daemon:      ", repr(self.options.daemon)
-        print "forever:     ", repr(self.options.forever)
-        print "sockname:    ", repr(self.options.sockname)
-        print "exitcodes:   ", repr(self.options.exitcodes)
-        print "user:        ", repr(self.options.user)
+        print("zdctl/zdrun options:")
+        print("schemafile:  ", repr(self.options.schemafile))
+        print("configfile:  ", repr(self.options.configfile))
+        print("zdrun:       ", repr(self.options.zdrun))
+        print("python:      ", repr(self.options.python))
+        print("program:     ", repr(self.options.program))
+        print("backofflimit:", repr(self.options.backofflimit))
+        print("daemon:      ", repr(self.options.daemon))
+        print("forever:     ", repr(self.options.forever))
+        print("sockname:    ", repr(self.options.sockname))
+        print("exitcodes:   ", repr(self.options.exitcodes))
+        print("user:        ", repr(self.options.user))
         umask = self.options.umask
         if not umask:
             # Here we're just getting the current umask so we can report it:
-            umask = os.umask(0777)
+            umask = os.umask(0o777)
             os.umask(umask)
-        print "umask:       ", oct(umask)
-        print "directory:   ", repr(self.options.directory)
-        print "logfile:     ", repr(self.options.logfile)
+        print("umask:       ", oct(umask))
+        print("directory:   ", repr(self.options.directory))
+        print("logfile:     ", repr(self.options.logfile))
+        print("transcript:  ", repr(self.options.transcript))
 
     def show_python(self):
-        print "Python info:"
+        print("Python info:")
         version = sys.version.replace("\n", "\n              ")
-        print "Version:     ", version
-        print "Platform:    ", sys.platform
-        print "Executable:  ", repr(sys.executable)
-        print "Arguments:   ", repr(sys.argv)
-        print "Directory:   ", repr(os.getcwd())
-        print "Path:"
+        print("Version:     ", version)
+        print("Platform:    ", sys.platform)
+        print("Executable:  ", repr(sys.executable))
+        print("Arguments:   ", repr(sys.argv))
+        print("Directory:   ", repr(os.getcwd()))
+        print("Path:")
         for dir in sys.path:
-            print "    " + repr(dir)
+            print("    " + repr(dir))
 
     def show_all(self):
         self.show_options()
-        print
+        print()
         self.show_python()
 
     def help_show(self):
-        print "show options -- show zdctl options"
-        print "show python -- show Python version and details"
-        print "show all -- show all of the above"
+        print("show options -- show zdctl options")
+        print("show python -- show Python version and details")
+        print("show all -- show all of the above")
 
     def do_logtail(self, arg):
         if not arg:
             arg = self.options.logfile
             if not arg:
-                print "No default log file specified; use logtail <logfile>"
+                print("No default log file specified; use logtail <logfile>")
                 return
         try:
             helper = TailHelper(arg)
             helper.tailf()
         except KeyboardInterrupt:
-            print
-        except IOError, msg:
-            print msg
-        except OSError, msg:
-            print msg
+            print()
+        except IOError as msg:
+            print(msg)
+        except OSError as msg:
+            print(msg)
 
     def help_logtail(self):
-        print "logtail [logfile] -- Run tail -f on the given logfile."
-        print "                     A default file may exist."
-        print "                     Hit ^C to exit this mode."
+        print("logtail [logfile] -- Run tail -f on the given logfile.")
+        print("                     A default file may exist.")
+        print("                     Hit ^C to exit this mode.")
 
     def do_foreground(self, arg):
         self.get_status()
         pid = self.zd_pid
         if pid:
-            print "To run the program in the foreground, please stop it first."
+            print("To run the program in the foreground, please stop it first.")
             return
 
         program = self.options.program + self.options.args[1:]
-        print " ".join(program)
+        print(" ".join(program))
         sys.stdout.flush()
         try:
             os.spawnlp(os.P_WAIT, program[0], *program)
         except KeyboardInterrupt:
-            print
+            print()
 
     def do_fg(self, arg):
         self.do_foreground(arg)
 
     def help_foreground(self):
-        print "foreground -- Run the program in the forground."
-        print "fg -- an alias for foreground."
+        print("foreground -- Run the program in the forground.")
+        print("fg -- an alias for foreground.")
 
     def help_fg(self):
         self.help_foreground()
@@ -559,7 +564,7 @@ class TailHelper:
             bytes_added = newsz - sz
             if bytes_added < 0:
                 sz = 0
-                print "==> File truncated <=="
+                print("==> File truncated <==")
                 bytes_added = newsz
             if bytes_added > 0:
                 self.f.seek(-bytes_added, 2)
